from django.conf import settings
from django.utils import unittest

from . import find_all_templates


class TemplateFinderTestMixin(object):

    def test_explicit_name(self):
        self.assertEqual(['404.html'], find_all_templates('404.html'))

    def test_wildcard_name(self):
        self.assertIn('403.html', find_all_templates('40*.html'))
        self.assertIn('404.html', find_all_templates('40*.html'))

    def test_subdirectory_wildcard(self):
        self.assertIn('menu/menu.html', find_all_templates('menu/*'))
        self.assertIn('menu/submenu.html', find_all_templates('menu/*'))


class AppDirectoriesLoaderTest(TemplateFinderTestMixin, unittest.TestCase):

    def setUp(self):
        settings.TEMPLATE_LOADERS = (
            'django.template.loaders.app_directories.Loader',
        )


class CachedLoaderTest(TemplateFinderTestMixin, unittest.TestCase):

    def setUp(self):
        settings.TEMPLATE_LOADERS = (
            'django.template.loaders.cached.Loader', (
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
            )
        )


class FilesystemLoaderTest(TemplateFinderTestMixin, unittest.TestCase):

    def setUp(self):
        settings.TEMPLATE_LOADERS = (
            'django.template.loaders.filesystem.Loader',
        )



import fnmatch
import logging
import os

from django.conf import settings
from django.utils.importlib import import_module


__all__ = ('find_all_templates',)


LOGGER = logging.getLogger('templatefinder')


def find_all_templates(pattern='*.html'):
    """
    Finds all Django templates matching given glob in all TEMPLATE_LOADERS

    :param str pattern: `glob <http://docs.python.org/2/library/glob.html>`_
                        to match

    .. important:: At the moment egg loader is not supported.
    """
    templates = []

    # See: https://docs.djangoproject.com/en/dev/ref/templates/api/#django.template.loaders.cached.Loader
    template_loaders = list(settings.TEMPLATE_LOADERS)
    if 'django.template.loaders.cached.Loader' in template_loaders:
        cached_loader_index = template_loaders.index('django.template.loaders.cached.Loader')
        extra_loaders = template_loaders[cached_loader_index + 1]
        # both the cached loader and the next element (list of regular loaders)
        del template_loaders[cached_loader_index]
        del template_loaders[cached_loader_index]
        for loader in extra_loaders:
            template_loaders.insert(cached_loader_index, loader)

    for loader_name in template_loaders:
        module, klass = loader_name.rsplit('.', 1)
        if loader_name in (
            'django.template.loaders.app_directories.Loader',
            'django.template.loaders.filesystem.Loader',
        ):
            loader = getattr(import_module(module), klass)()
            for dir in loader.get_template_sources(''):
                for root, dirnames, filenames in os.walk(dir):
                    for basename in filenames:
                        filename = os.path.join(root, basename)
                        rel_filename = filename[len(dir)+1:]
                        if fnmatch.fnmatch(filename, pattern) or \
                           fnmatch.fnmatch(basename, pattern) or \
                           fnmatch.fnmatch(rel_filename, pattern):
                            templates.append(rel_filename)
        else:
            LOGGER.debug('%s is not supported' % loader_name)
    return sorted(set(templates))



#!/usr/bin/env python

import os
from setuptools import setup, find_packages

import templatefinder


CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Internet :: WWW/HTTP',
]

setup(
    author='Piotr Kilczuk',
    author_email='piotr@tymaszweb.pl',
    name='django-template-finder',
    version='.'.join(str(v) for v in templatefinder.VERSION),
    description='Simple Django utility that allows you to find templates with names matching given pattern',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    url='https://github.com/TyMaszWeb/django-template-finder',
    license='BSD License',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    install_requires=[],
    tests_require=[
        'tox>=1.4.3',
    ],
    packages=find_packages(),
    include_package_data=False,
    zip_safe=False,
    test_suite='runtests.main',
)
